@@ -981,5 +981,5 @@ void CmdLineParser::PrintHelp()
               "  cppcheck -I inc1/ -I inc2/ f.cpp\n"
               "\n"
               "For more information:\n"
-              "    http://cppcheck.sourceforge.net/manual.pdf\n";
+              "    http://cppcheck.net/manual.pdf\n";
 }
