@@ -31,6 +31,30 @@
 class ErrorLogger;
 class Settings;
 
+/**
+ * @brief A preprocessor directive
+ * Each preprocessor directive (\#include, \#define, \#undef, \#if, \#ifdef, \#else, \#endif)
+ * will be recorded as an instance of this class.
+ *
+ * file and linenr denote the location where where the directive is defined.
+ *
+ */
+
+class CPPCHECKLIB Directive {
+public:
+    /** name of (possibly included) file where directive is defined */
+    std::string file;
+
+    /** line number in (possibly included) file where directive is defined */
+    int linenr;
+
+    /** the actual directive text */
+    std::string str;
+
+    /** record a directive (possibly filtering src) */
+    Directive(const std::string &_file, const int _linenr, const std::string &_str);
+};
+
 /// @addtogroup Core
 /// @{
 
@@ -250,6 +274,11 @@ public:
         file0 = f;
     }
 
+    /**
+     * dump all directives present in source file
+     */
+    void dump(std::ostream &out) const;
+
 private:
     void missingInclude(const std::string &filename, unsigned int linenr, const std::string &header, HeaderTypes headerType);
 
@@ -271,6 +300,9 @@ private:
     Settings& _settings;
     ErrorLogger *_errorLogger;
 
+    /** list of all directives met while preprocessing file */
+    std::list<Directive> directives;
+
     /** filename for cpp/c file - useful when reporting errors */
     std::string file0;
 };
