@@ -77,16 +77,19 @@ private:
         TEST_CASE(clarifyCondition4);     // ticket #3110
         TEST_CASE(clarifyCondition5);     // #3609 CWinTraits<WS_CHILD|WS_VISIBLE>..
         TEST_CASE(clarifyCondition6);     // #3818
+        TEST_CASE(clarifyCondition7);
 
         TEST_CASE(alwaysTrue);
 
         TEST_CASE(checkInvalidTestForOverflow);
     }
 
-    void check(const char code[], const char* filename = "test.cpp") {
+    void check(const char code[], const char* filename = "test.cpp", bool inconclusive = false) {
         // Clear the error buffer..
         errout.str("");
 
+        settings0.inconclusive = inconclusive;
+
         CheckCondition checkCondition;
 
         // Tokenize..
@@ -945,13 +948,28 @@ private:
     void incorrectLogicOperator6() { // char literals
         check("void f(char x) {\n"
               "  if (x == '1' || x == '2') {}\n"
-              "}");
+              "}", "test.cpp", true);
         ASSERT_EQUALS("", errout.str());
 
         check("void f(char x) {\n"
               "  if (x == '1' && x == '2') {}\n"
-              "}");
-        TODO_ASSERT_EQUALS("error", "", errout.str());
+              "}", "test.cpp", true);
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Logical conjunction always evaluates to false: x == '1' && x == '2'.\n", errout.str());
+
+        check("int f(char c) {\n"
+              "  return (c >= 'a' && c <= 'z');\n"
+              "}", "test.cpp", true);
+        ASSERT_EQUALS("", errout.str());
+
+        check("int f(char c) {\n"
+              "  return (c <= 'a' && c >= 'z');\n"
+              "}", "test.cpp", true);
+        ASSERT_EQUALS("[test.cpp:2]: (warning, inconclusive) Logical conjunction always evaluates to false: c <= 'a' && c >= 'z'.\n", errout.str());
+
+        check("int f(char c) {\n"
+              "  return (c <= 'a' && c >= 'z');\n"
+              "}", "test.cpp", false);
+        ASSERT_EQUALS("", errout.str());
     }
 
     void incorrectLogicOperator7() { // opposite expressions
@@ -1499,12 +1517,14 @@ private:
         check("void f() {\n"
               "    if (x & 3 == 2) {}\n"
               "}");
-        ASSERT_EQUALS("[test.cpp:2]: (style) Suspicious condition (bitwise operator + comparison); Clarify expression with parentheses.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:2]: (style) Suspicious condition (bitwise operator + comparison); Clarify expression with parentheses.\n"
+                      "[test.cpp:2]: (style) Boolean result is used in bitwise operation. Clarify expression with parentheses.\n", errout.str());
 
         check("void f() {\n"
               "    if (a & fred1.x == fred2.y) {}\n"
               "}");
-        ASSERT_EQUALS("[test.cpp:2]: (style) Suspicious condition (bitwise operator + comparison); Clarify expression with parentheses.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:2]: (style) Suspicious condition (bitwise operator + comparison); Clarify expression with parentheses.\n"
+                      "[test.cpp:2]: (style) Boolean result is used in bitwise operation. Clarify expression with parentheses.\n", errout.str());
     }
 
 // clarify condition that uses ! operator and then bitwise operator
@@ -1566,6 +1586,15 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void clarifyCondition7() {
+        // Ensure that binary and unary &, and & in declarations are distinguished properly
+        check("void f(bool error) {\n"
+              "    bool & withoutSideEffects=found.first->second;\n" // Declaring a reference to a boolean; & is no operator at all
+              "    execute(secondExpression, &programMemory, &result, &error);\n" // Unary &
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void testBug5895() {
         check("void png_parse(uint64_t init, int buf_size) {\n"
               "    if (init == 0x89504e470d0a1a0a || init == 0x8a4d4e470d0a1a0a)\n"
@@ -1622,6 +1651,29 @@ private:
               "  assert(x == 0);\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        //TRAC #7428 false negative: Statement is always false
+        check("void f() {\n"
+              "assert( (a & 0x07) == 8U );\n" // statement always false, because 7 == 8 is false
+              "assert( (a & 0x07) >  7U );\n" // statement always false, because 7 > 7 is false
+              "assert( (a | 0x07) <  7U );\n" // statement always false, because 7 < 7 is false
+              "assert( (a & 0x07) >  8U );\n" // statement always false, because 7 > 8 is false
+              "assert( (a | 0x07) <  6U );\n" // statement always false, because 7 < 6 is false
+              "assert( (a & 0x07) >= 7U );\n" // statement correct
+              "assert( (a | 0x07) <= 7U );\n" // statement correct
+              "assert( (a & 0x07) >= 8U );\n" // statement always false, because 7 >= 8 is false
+              "assert( (a | 0x07) <= 6U );\n" // statement always false, because 7 <= 6 is false
+              "assert( (a & 0x07) >  3U );\n" // statement correct
+              "assert( (a | 0x07) <  9U );\n" // statement correct
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Expression '(X & 0x7) == 0x8' is always false.\n"
+                      "[test.cpp:3]: (style) Expression '(X & 0x7) > 0x7' is always false.\n"
+                      "[test.cpp:4]: (style) Expression '(X | 0x7) < 0x7' is always false.\n"
+                      "[test.cpp:5]: (style) Expression '(X & 0x7) > 0x8' is always false.\n"
+                      "[test.cpp:6]: (style) Expression '(X | 0x7) < 0x6' is always false.\n"
+                      "[test.cpp:9]: (style) Expression '(X & 0x7) >= 0x8' is always false.\n"
+                      "[test.cpp:10]: (style) Expression '(X | 0x7) <= 0x6' is always false.\n",
+                      errout.str());
     }
 
     void checkInvalidTestForOverflow() {
