@@ -230,6 +230,8 @@ private:
         TEST_CASE(garbageCode179); // #3533
         TEST_CASE(garbageCode180);
         TEST_CASE(garbageCode181);
+        TEST_CASE(garbageCode182); // #4195
+        TEST_CASE(garbageCode183); // #7505
         TEST_CASE(garbageValueFlow);
         TEST_CASE(garbageSymbolDatabase);
         TEST_CASE(garbageAST);
@@ -387,11 +389,11 @@ private:
     }
 
     void garbageCode2() { //#4300 (segmentation fault)
-        ASSERT_THROW(checkCode("enum { D = 1  struct  { } ; }  s.b = D;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { D = 1  struct  { } ; }  s.b = D;"), InternalError);
     }
 
     void garbageCode3() { //#4849 (segmentation fault in Tokenizer::simplifyStructDecl (invalid code))
-        ASSERT_THROW(checkCode("enum {  D = 2 s ; struct y  { x } ; } { s.a = C ; s.b = D ; }"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum {  D = 2 s ; struct y  { x } ; } { s.a = C ; s.b = D ; }"), InternalError);
     }
 
     void garbageCode4() { // #4887
@@ -413,19 +415,19 @@ private:
     }
 
     void garbageCode8() { // #5604
-        ASSERT_THROW(checkCode("{ enum struct : };"), InternalError);
-        ASSERT_THROW(checkCode("int ScopedEnum{ template<typename T> { { e = T::error }; };\n"
-                               "ScopedEnum1<int> se1; { enum class E : T { e = 0 = e ScopedEnum2<void*> struct UnscopedEnum3 { T{ e = 4 }; };\n"
-                               "arr[(int) E::e]; }; UnscopedEnum3<int> e2 = f()\n"
-                               "{ { e = e1; T::error } int test1 ue2; g() { enum class E { e = T::error }; return E::e; } int test2 = } \n"
-                               "namespace UnscopedEnum { template<typename T> struct UnscopedEnum1 { E{ e = T::error }; }; UnscopedEnum1<int> { enum E : { e = 0 }; };\n"
-                               "UnscopedEnum2<void*> ue3; template<typename T> struct UnscopedEnum3 { enum { }; }; int arr[E::e]; };\n"
-                               "UnscopedEnum3<int> namespace template<typename T> int f() { enum E { e }; T::error }; return (int) E(); } int test1 int g() { enum E { e = E };\n"
-                               "E::e; } int test2 = g<int>(); }"), InternalError);
+        TODO_ASSERT_THROW(checkCode("{ enum struct : };"), InternalError);
+        TODO_ASSERT_THROW(checkCode("int ScopedEnum{ template<typename T> { { e = T::error }; };\n"
+                                    "ScopedEnum1<int> se1; { enum class E : T { e = 0 = e ScopedEnum2<void*> struct UnscopedEnum3 { T{ e = 4 }; };\n"
+                                    "arr[(int) E::e]; }; UnscopedEnum3<int> e2 = f()\n"
+                                    "{ { e = e1; T::error } int test1 ue2; g() { enum class E { e = T::error }; return E::e; } int test2 = } \n"
+                                    "namespace UnscopedEnum { template<typename T> struct UnscopedEnum1 { E{ e = T::error }; }; UnscopedEnum1<int> { enum E : { e = 0 }; };\n"
+                                    "UnscopedEnum2<void*> ue3; template<typename T> struct UnscopedEnum3 { enum { }; }; int arr[E::e]; };\n"
+                                    "UnscopedEnum3<int> namespace template<typename T> int f() { enum E { e }; T::error }; return (int) E(); } int test1 int g() { enum E { e = E };\n"
+                                    "E::e; } int test2 = g<int>(); }"), InternalError);
     }
 
     void garbageCode9() {
-        ASSERT_THROW(checkCode("enum { e = { } } ( ) { { enum { } } } { e } "), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { e = { } } ( ) { { enum { } } } { e } "), InternalError);
     }
 
     void garbageCode10() { // #6127
@@ -896,7 +898,7 @@ private:
     }
 
     void garbageCode107() { // #6881
-        ASSERT_THROW(checkCode("enum { val = 1{ }; { const} }; { } Bar { const int A = val const } ;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { val = 1{ }; { const} }; { } Bar { const int A = val const } ;"), InternalError);
     }
 
     void garbageCode108() { //  #6895 "segmentation fault (invalid code) in CheckCondition::isOppositeCond"
@@ -912,11 +914,11 @@ private:
     }
 
     void garbageCode111() { //  #6907
-        ASSERT_THROW(checkCode("enum { FOO = 1( ,) } {{ FOO }} ;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { FOO = 1( ,) } {{ FOO }} ;"), InternalError);
     }
 
     void garbageCode112() { //  #6909
-        ASSERT_THROW(checkCode("enum { FOO = ( , ) } {{ }}>> enum { FOO< = ( ) } { { } } ;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { FOO = ( , ) } {{ }}>> enum { FOO< = ( ) } { { } } ;"), InternalError);
     }
 
     void garbageCode113() { //  #6858
@@ -939,8 +941,8 @@ private:
     }
 
     void garbageCode117() { // #6121
-        ASSERT_THROW(checkCode("enum E { f = {} };\n"
-                               "int a = f;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum E { f = {} };\n"
+                                    "int a = f;"), InternalError);
     }
 
     void garbageCode118() { // #5600 - missing include causes invalid enum
@@ -1014,8 +1016,8 @@ private:
     }
 
     void garbageCode128() {
-        ASSERT_THROW(checkCode("enum { FOO = ( , ) } {{ }} enum {{ FOO << = } ( ) } {{ }} ;"),
-                     InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { FOO = ( , ) } {{ }} enum {{ FOO << = } ( ) } {{ }} ;"),
+                          InternalError);
     }
 
     void garbageCode129() {
@@ -1024,8 +1026,8 @@ private:
     }
 
     void garbageCode130() {
-        ASSERT_THROW(checkCode("enum { FOO = ( , ){ } { { } } { { FOO} = } ( ) } { { } } enumL\" ( enumL\" { { FOO } ( ) } { { } } ;"),
-                     InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { FOO = ( , ){ } { { } } { { FOO} = } ( ) } { { } } enumL\" ( enumL\" { { FOO } ( ) } { { } } ;"),
+                          InternalError);
     }
 
     void garbageCode131() {
@@ -1149,7 +1151,7 @@ private:
     }
 
     void garbageCode141() { // #7043
-        ASSERT_THROW(checkCode("enum { X = << { X } } enum { X = X } = X ;"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { X = << { X } } enum { X = X } = X ;"), InternalError);
     }
 
     void garbageCode142() { // #7050
@@ -1241,7 +1243,7 @@ private:
     }
 
     void garbageCode153() {
-        ASSERT_THROW(checkCode("enum { X = << { X } } { X X } enum { X = << { ( X ) } } { } X */"), InternalError);
+        TODO_ASSERT_THROW(checkCode("enum { X = << { X } } { X X } enum { X = << { ( X ) } } { } X */"), InternalError);
     }
 
     void garbageCode154() {
@@ -1509,6 +1511,16 @@ private:
     void garbageCode181() {
         checkCode("int test() { int +; }");
     }
+
+    // #4195 - segfault for "enum { int f ( ) { return = } r = f ( ) ; }"
+    void garbageCode182() {
+        ASSERT_THROW(checkCode("enum { int f ( ) { return = } r = f ( ) ; }"), InternalError);
+    }
+    // #7505 - segfault
+    void garbageCode183() {
+        ASSERT_THROW(checkCode("= { int } enum return { r = f() f(); }"), InternalError);
+    }
+
 };
 
 REGISTER_TEST(TestGarbage)
