@@ -44,7 +44,6 @@ private:
         TEST_CASE(strPlusChar1);     // "/usr" + '/'
         TEST_CASE(strPlusChar2);     // "/usr" + ch
         TEST_CASE(strPlusChar3);     // ok: path + "/sub" + '/'
-        TEST_CASE(strPlusChar4);     // ast
 
         TEST_CASE(sprintf1);        // Dangerous usage of sprintf
         TEST_CASE(sprintf2);
@@ -155,6 +154,16 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
+        check("void f() {\n"
+              "  if (strcmp($\"00FF00\", \"00FF00\") == 0) {}"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "  if ($strcmp(\"00FF00\", \"00FF00\") == 0) {}"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         check("int main()\n"
               "{\n"
               "  if (stricmp(\"hotdog\",\"HOTdog\") == 0)"
@@ -474,11 +483,6 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void strPlusChar4() {
-        // don't crash
-        check("int test() { int +; }");
-    }
-
 
     void incorrectStringCompare() {
         check("int f() {\n"
